package queryingdatastore;

import java.util.*;
import org.hibernate.*;
import org.hibernate.cfg.*;
import org.hibernate.criterion.*;

public class Main {

    public static void main(String[] args) {
        Koszyk k1 = new Koszyk();
        k1.setKlient("Jan Kowalski");
        for (int i = 0; i < 50; i++) {
            Produkt p = new Produkt();
            p.setNazwa("Banan"+i);
            p.setCena(i+100);
            k1.getProdukty().add(p);
            p.setKoszyk(k1);
        }
        Koszyk k2 = new Koszyk();
        k2.setKlient("John Maliniak");
        for (int i = 0; i < 50; i++) {
            Produkt p = new Produkt();
            p.setNazwa("Serek"+i);
            p.setCena(i+100);
            k2.getProdukty().add(p);
            p.setKoszyk(k2);
        }

        Session session1;
        SessionFactory sessionFactory = new Configuration().configure().buildSessionFactory();
        session1 = sessionFactory.openSession();
        Transaction tx1 = session1.beginTransaction();
        session1.save(k1);
        session1.save(k2);
        tx1.commit();

        // Natywny SQL
        System.out.println("------- NATYWNY SQL -------");
        Transaction tx2 = session1.beginTransaction();
        List<Produkt> produkty = session1.createSQLQuery("select {p.*} from PRODUKT {p}")
                .addEntity("p", Produkt.class).list();
        for (Produkt p : produkty)
        {
            System.out.println(p.getNazwa()+" "+p.getCena());
        }
        tx2.commit();
        
        // Stronicowanie 1
        System.out.println("------- STRONICOWANIE 1 -------");
        Transaction tx3 = session1.beginTransaction();
        Query query1 = session1.createQuery("from Produkt p order by p.Nazwa asc");
        query1.setFirstResult(30);
        query1.setMaxResults(5);
        for (Object o : query1.list())
        {
            System.out.println(((Produkt)o).getNazwa());
        }
        tx3.commit();

        // Stronicowanie 2
        System.out.println("------- STRONICOWANIE 2 -------");
        Transaction tx4 = session1.beginTransaction();
        List<Produkt> produkty2 = session1.createQuery("from Produkt p order by p.Nazwa asc")
                .setFirstResult(30)
                .setMaxResults(5)
                .list();
        for (Produkt p : produkty2)
        {
            System.out.println(p.getNazwa());
        }
        tx4.commit();

        // Stronicowanie 3
        System.out.println("------- STRONICOWANIE 3 -------");
        Transaction tx5 = session1.beginTransaction();
        Criteria crit = session1.createCriteria(Produkt.class);
        crit.addOrder(Order.asc("Nazwa"));
        crit.setFirstResult(20);
        crit.setMaxResults(4);
        for (Object o : crit.list())
        {
            System.out.println(((Produkt)o).getNazwa());
        }
        tx5.commit();

        // Stronicowanie 4
        System.out.println("------- STRONICOWANIE 4 -------");
        Transaction tx6 = session1.beginTransaction();
        List<Produkt> produkty3 = session1.createCriteria(Produkt.class)
                .addOrder(Order.asc("Nazwa"))
                .setFirstResult(20)
                .setMaxResults(4)
                .list();
        for (Produkt p : produkty3)
        {
            System.out.println(p.getNazwa());
        }
        tx6.commit();

        // Dowiązanie parametrów pozycyjnych
        System.out.println("------- PARAMETRY POZYCYJNE i NAZWANE -------");
        Transaction tx7 = session1.beginTransaction();
        List<Produkt> produkty4 = session1.createQuery("from Produkt p where Id between ? and ? and Nazwa like :nazwa order by p.Nazwa asc")
                .setInteger(0, 1)
                .setInteger(1, 10)
                .setString("nazwa", "Banan__")
                .list();
        for (Produkt p : produkty4)
        {
            System.out.println(p.getNazwa());
        }
        tx7.commit();

        // Proste zapytanie
        System.out.println("------- PROSTE ZAPYTANIE 1 -------");
        Transaction tx8 = session1.beginTransaction();
        List<Koszyk> koszyki = session1.createQuery("from Koszyk k").list();
        for (Koszyk k : koszyki)
        {
            System.out.println(k.getKlient()+", liczba towarów "+k.getProdukty().size());
        }
        tx8.commit();

        // Mniej proste zapytanie
        System.out.println("------- PROSTE ZAPYTANIE 2 -------");
        Transaction tx9 = session1.beginTransaction();
        List<Produkt> produkty5 = session1.createCriteria(Produkt.class)
                .createAlias("Koszyk", "Koszyk")
                .add(Expression.or(
                    Expression.le("Cena", new Double(110)),
                    Expression.ge("Cena", new Double(140))
                ))
                .add(Expression.like("Koszyk.Klient", "owal", MatchMode.ANYWHERE).ignoreCase())
                .addOrder(Order.asc("Cena"))
                .list();
        System.out.println(produkty5.size());
        for (Produkt p : produkty5)
        {
            System.out.println(p.getNazwa()+" "+p.getCena());
        }
        tx9.commit();

        
        session1.close();
    }
}
